@@ -6,7 +6,8 @@
 
 use crate::process_state::*;
 use crate::stackwalker::walk_stack;
-use crate::{string_symbol_supplier, Symbolizer};
+use crate::{string_symbol_supplier, Symbolizer, SystemInfo};
+use minidump::system_info::{Cpu, Os};
 use minidump::*;
 use std::collections::HashMap;
 use test_assembler::*;
@@ -33,7 +34,7 @@ impl TestFixture {
         }
     }
 
-    pub fn walk_stack(&self, stack: Section) -> CallStack {
+    pub async fn walk_stack(&self, stack: Section) -> CallStack {
         let context = MinidumpContext {
             raw: MinidumpRawContext::Arm64(self.raw.clone()),
             valid: MinidumpContextValidity::All,
@@ -47,13 +48,24 @@ impl TestFixture {
             size,
             bytes: &stack,
         };
+        let system_info = SystemInfo {
+            os: Os::Windows,
+            os_version: None,
+            os_build: None,
+            cpu: Cpu::Arm64,
+            cpu_info: None,
+            cpu_microcode_version: None,
+            cpu_count: 1,
+        };
         let symbolizer = Symbolizer::new(string_symbol_supplier(self.symbols.clone()));
         walk_stack(
             &Some(&context),
             Some(&stack_memory),
             &self.modules,
+            &system_info,
             &symbolizer,
         )
+        .await
     }
 
     pub fn add_symbols(&mut self, name: String, symbols: String) {
@@ -61,8 +73,8 @@ impl TestFixture {
     }
 }
 
-#[test]
-fn test_simple() {
+#[tokio::test]
+async fn test_simple() {
     let mut f = TestFixture::new();
     let stack = Section::new();
     stack.start().set_const(0x80000000);
@@ -73,15 +85,15 @@ fn test_simple() {
     f.raw.set_register("pc", 0x4000c020);
     f.raw.set_register("fp", 0x80000000);
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 1);
     let f = &s.frames[0];
     let m = f.module.as_ref().unwrap();
     assert_eq!(m.code_file(), "module1");
 }
 
-#[test]
-fn test_scan_without_symbols() {
+#[tokio::test]
+async fn test_scan_without_symbols() {
     // Scanning should work without any symbols
     let mut f = TestFixture::new();
     let mut stack = Section::new();
@@ -111,7 +123,7 @@ fn test_scan_without_symbols() {
     f.raw.set_register("pc", 0x40005510);
     f.raw.set_register("sp", stack.start().value().unwrap());
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 3);
 
     {
@@ -166,8 +178,8 @@ fn test_scan_without_symbols() {
     }
 }
 
-#[test]
-fn test_scan_with_symbols() {
+#[tokio::test]
+async fn test_scan_with_symbols() {
     // Test that we can refine our scanning using symbols. Specifically we
     // should be able to reject pointers that are in modules but don't map to
     // any FUNC/PUBLIC record.
@@ -205,7 +217,7 @@ fn test_scan_with_symbols() {
         String::from("FUNC 100 400 10 marsupial\n"),
     );
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 2);
 
     {
@@ -238,8 +250,8 @@ fn test_scan_with_symbols() {
     }
 }
 
-#[test]
-fn test_scan_first_frame() {
+#[tokio::test]
+async fn test_scan_first_frame() {
     // The first (context) frame gets extra long scans, this test checks that.
     let mut f = TestFixture::new();
     let mut stack = Section::new();
@@ -271,7 +283,7 @@ fn test_scan_first_frame() {
     f.raw.set_register("pc", 0x40005510);
     f.raw.set_register("sp", stack.start().value().unwrap());
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 2);
 
     {
@@ -304,8 +316,8 @@ fn test_scan_first_frame() {
     }
 }
 
-#[test]
-fn test_frame_pointer() {
+#[tokio::test]
+async fn test_frame_pointer() {
     // Frame-pointer-based unwinding
     let mut f = TestFixture::new();
     let mut stack = Section::new();
@@ -345,7 +357,7 @@ fn test_frame_pointer() {
     f.raw.set_register("fp", frame1_fp.value().unwrap());
     f.raw.set_register("sp", stack.start().value().unwrap());
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 3);
 
     {
@@ -407,8 +419,8 @@ fn test_frame_pointer() {
     }
 }
 
-#[test]
-fn test_ptr_auth_strip() {
+#[tokio::test]
+async fn test_ptr_auth_strip() {
     // Same as the basic frame pointer test but extra high bits have been set which
     // must be masked out. This is vaguely emulating Arm Pointer Authentication,
     // although very synthetically. This might break if we implement more accurate
@@ -455,7 +467,7 @@ fn test_ptr_auth_strip() {
     f.raw.set_register("fp", frame1_fp.value().unwrap());
     f.raw.set_register("sp", stack.start().value().unwrap());
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 3);
 
     {
@@ -590,13 +602,13 @@ fn init_cfi_state() -> (TestFixture, Section, Context, MinidumpContextValidity)
     (f, stack, expected, expected_valid)
 }
 
-fn check_cfi(
+async fn check_cfi(
     f: TestFixture,
     stack: Section,
     expected: Context,
     expected_valid: MinidumpContextValidity,
 ) {
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 2);
 
     {
@@ -634,8 +646,8 @@ fn check_cfi(
     unreachable!();
 }
 
-#[test]
-fn test_cfi_at_4000() {
+#[tokio::test]
+async fn test_cfi_at_4000() {
     let (mut f, mut stack, expected, expected_valid) = init_cfi_state();
 
     stack = stack.append_repeated(0, 120);
@@ -643,11 +655,11 @@ fn test_cfi_at_4000() {
     f.raw.set_register("pc", 0x0000000040004000);
     f.raw.set_register("lr", 0x0000000040005510);
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4001() {
+#[tokio::test]
+async fn test_cfi_at_4001() {
     let (mut f, mut stack, mut expected, expected_valid) = init_cfi_state();
 
     let frame1_sp = Label::new();
@@ -665,11 +677,11 @@ fn test_cfi_at_4001() {
     f.raw.set_register("x20", 0x623135ac35ac6231);
     f.raw.set_register("fp", 0x5fc4be14be145fc4);
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4002() {
+#[tokio::test]
+async fn test_cfi_at_4002() {
     let (mut f, mut stack, mut expected, expected_valid) = init_cfi_state();
 
     let frame1_sp = Label::new();
@@ -693,11 +705,11 @@ fn test_cfi_at_4002() {
     f.raw.iregs[22] = 0x2561562f562f2561; // distinct callee x22
     f.raw.set_register("fp", 0x5fc4be14be145fc4);
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4003() {
+#[tokio::test]
+async fn test_cfi_at_4003() {
     let (mut f, mut stack, mut expected, mut expected_valid) = init_cfi_state();
 
     let frame1_sp = Label::new();
@@ -722,11 +734,11 @@ fn test_cfi_at_4003() {
     f.raw.iregs[1] = 0xfb756319fb756319;
     f.raw.set_register("fp", 0x5fc4be14be145fc4);
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4004() {
+#[tokio::test]
+async fn test_cfi_at_4004() {
     // Should just be the same as 4003
 
     let (mut f, mut stack, mut expected, mut expected_valid) = init_cfi_state();
@@ -753,11 +765,11 @@ fn test_cfi_at_4004() {
     f.raw.iregs[1] = 0xfb756319fb756319;
     f.raw.set_register("fp", 0x5fc4be14be145fc4);
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4005() {
+#[tokio::test]
+async fn test_cfi_at_4005() {
     // Here we move the .cfa, but provide an explicit rule to recover the SP,
     // so again there should be no change in the registers recovered.
 
@@ -784,11 +796,11 @@ fn test_cfi_at_4005() {
     f.raw.set_register("pc", 0x0000000040004005);
     f.raw.iregs[1] = 0xfb756319fb756319;
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_at_4006() {
+#[tokio::test]
+async fn test_cfi_at_4006() {
     // Here we provide an explicit rule for the PC, and have the saved .ra be
     // bogus.
 
@@ -816,11 +828,11 @@ fn test_cfi_at_4006() {
     f.raw.set_register("pc", 0x0000000040004006);
     f.raw.iregs[1] = 0xfb756319fb756319;
 
-    check_cfi(f, stack, expected, expected_valid);
+    check_cfi(f, stack, expected, expected_valid).await;
 }
 
-#[test]
-fn test_cfi_reject_backwards() {
+#[tokio::test]
+async fn test_cfi_reject_backwards() {
     // Check that we reject rules that would cause the stack pointer to
     // move in the wrong direction.
 
@@ -832,12 +844,12 @@ fn test_cfi_reject_backwards() {
     f.raw.set_register("sp", 0x0000000080000000);
     f.raw.set_register("lr", 0x0000000040005510);
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 1);
 }
 
-#[test]
-fn test_cfi_reject_bad_exprs() {
+#[tokio::test]
+async fn test_cfi_reject_bad_exprs() {
     // Check that we reject rules whose expressions' evaluation fails.
 
     let (mut f, mut stack, _expected, _expected_valid) = init_cfi_state();
@@ -847,6 +859,218 @@ fn test_cfi_reject_bad_exprs() {
     f.raw.set_register("pc", 0x0000000040007000);
     f.raw.set_register("sp", 0x0000000080000000);
 
-    let s = f.walk_stack(stack);
+    let s = f.walk_stack(stack).await;
     assert_eq!(s.frames.len(), 1);
 }
+
+#[tokio::test]
+async fn test_frame_pointer_overflow() {
+    // Make sure we don't explode when trying frame pointer analysis on a value
+    // that will overflow.
+
+    type Pointer = u64;
+    let stack_max: Pointer = Pointer::MAX;
+    let stack_size: Pointer = 1000;
+    let bad_frame_ptr: Pointer = stack_max;
+
+    let mut f = TestFixture::new();
+    let mut stack = Section::new();
+    let stack_start: Pointer = stack_max - stack_size;
+    stack.start().set_const(stack_start as u64);
+
+    stack = stack
+        // frame 0
+        .append_repeated(0, stack_size as usize); // junk, not important to the test
+
+    f.raw.set_register("pc", 0x00007400c0000200);
+    f.raw.set_register("fp", bad_frame_ptr);
+    f.raw
+        .set_register("sp", stack.start().value().unwrap() as Pointer);
+    f.raw.set_register("lr", 0x00007500b0000110);
+
+    let s = f.walk_stack(stack).await;
+    assert_eq!(s.frames.len(), 1);
+
+    // As long as we don't panic, we're good!
+}
+
+#[tokio::test]
+async fn test_frame_pointer_barely_no_overflow() {
+    // This is a simple frame pointer test but with the all the values pushed
+    // as close to the upper memory boundary as possible, to confirm that
+    // our code doesn't randomly overflow *AND* isn't overzealous in
+    // its overflow guards.
+
+    let mut f = TestFixture::new();
+    let mut stack = Section::new();
+
+    type Pointer = u64;
+    let stack_max: Pointer = Pointer::MAX;
+    let pointer_size: Pointer = std::mem::size_of::<Pointer>() as Pointer;
+    let stack_size: Pointer = pointer_size * 3;
+
+    let stack_start: Pointer = stack_max - stack_size;
+    let return_address: Pointer = 0x00007500b0000110;
+    stack.start().set_const(stack_start as u64);
+
+    let frame0_fp = Label::new();
+    let frame1_sp = Label::new();
+    let frame1_fp = Label::new();
+
+    stack = stack
+        // frame 0
+        .mark(&frame0_fp)
+        .D64(&frame1_fp) // caller-pushed %rbp
+        .D64(return_address) // actual return address
+        // frame 1
+        .mark(&frame1_sp)
+        .mark(&frame1_fp) // end of stack
+        .D64(0);
+
+    f.raw.set_register("pc", 0x00007400c0000200);
+    f.raw
+        .set_register("fp", frame0_fp.value().unwrap() as Pointer);
+    f.raw
+        .set_register("sp", stack.start().value().unwrap() as Pointer);
+    f.raw.set_register("lr", return_address);
+
+    let s = f.walk_stack(stack).await;
+    assert_eq!(s.frames.len(), 2);
+
+    {
+        // Frame 0
+        let frame = &s.frames[0];
+        let valid = &frame.context.valid;
+        assert_eq!(frame.trust, FrameTrust::Context);
+        assert_eq!(frame.context.valid, MinidumpContextValidity::All);
+
+        if let MinidumpRawContext::Arm64(ctx) = &frame.context.raw {
+            assert_eq!(
+                ctx.get_register("fp", valid).unwrap(),
+                frame0_fp.value().unwrap() as Pointer
+            );
+        } else {
+            unreachable!();
+        }
+    }
+
+    {
+        // Frame 1
+        let frame = &s.frames[1];
+        let valid = &frame.context.valid;
+        assert_eq!(frame.trust, FrameTrust::FramePointer);
+        if let MinidumpContextValidity::Some(ref which) = valid {
+            assert_eq!(which.len(), 4);
+        } else {
+            unreachable!();
+        }
+
+        if let MinidumpRawContext::Arm64(ctx) = &frame.context.raw {
+            assert_eq!(ctx.get_register("pc", valid).unwrap(), return_address);
+            assert_eq!(
+                ctx.get_register("sp", valid).unwrap(),
+                frame1_sp.value().unwrap() as Pointer
+            );
+            assert_eq!(
+                ctx.get_register("fp", valid).unwrap(),
+                frame1_fp.value().unwrap() as Pointer
+            );
+            assert_eq!(ctx.get_register("lr", valid).unwrap(), return_address);
+        } else {
+            unreachable!();
+        }
+    }
+}
+
+#[tokio::test]
+async fn test_frame_pointer_infinite_equality() {
+    // Leaf functions on Arm are allowed to not update the stack pointer, so
+    // it's valid for the frame pointer analysis to conclude that the stack
+    // pointer doesn't change. However we must only provide this allowance
+    // to the first stack frame, or else we're vulnerable to infinite loops.
+    //
+    // One of the CFI tests already checks that we allow the leaf case to work,
+    // so here we test that we don't get stuck in an infinite loop for the
+    // non-leaf case.
+    //
+    // This is just a copy-paste of test_frame_pointer except for the line
+    // "EVIL INFINITE FRAME POINTER" has been changed from frame2_fp to frame1_fp.
+
+    // Frame-pointer-based unwinding
+    let mut f = TestFixture::new();
+    let mut stack = Section::new();
+    stack.start().set_const(0x80000000);
+
+    let return_address1 = 0x50000100u64;
+    let return_address2 = 0x50000900u64;
+    let frame1_sp = Label::new();
+    let frame2_sp = Label::new();
+    let frame1_fp = Label::new();
+    let frame2_fp = Label::new();
+
+    stack = stack
+        // frame 0
+        .append_repeated(0, 64) // space
+        .D64(0x0000000D) // junk that's not
+        .D64(0xF0000000) // a return address
+        .mark(&frame1_fp) // next fp will point to the next value
+        .D64(&frame1_fp) // <--- EVIL INFINITE FRAME POINTER
+        .D64(return_address2) // save current link register
+        .mark(&frame1_sp)
+        // frame 1
+        .append_repeated(0, 64) // space
+        .D64(0x0000000D) // junk that's not
+        .D64(0xF0000000) // a return address
+        .mark(&frame2_fp)
+        .D64(0)
+        .D64(0)
+        .mark(&frame2_sp)
+        // frame 2
+        .append_repeated(0, 64) // Whatever values on the stack.
+        .D64(0x0000000D) // junk that's not
+        .D64(0xF0000000); // a return address.
+
+    f.raw.set_register("pc", 0x40005510);
+    f.raw.set_register("lr", return_address1);
+    f.raw.set_register("fp", frame1_fp.value().unwrap());
+    f.raw.set_register("sp", stack.start().value().unwrap());
+
+    let s = f.walk_stack(stack).await;
+    assert_eq!(s.frames.len(), 2);
+
+    {
+        // Frame 0
+        let frame = &s.frames[0];
+        assert_eq!(frame.trust, FrameTrust::Context);
+        assert_eq!(frame.context.valid, MinidumpContextValidity::All);
+    }
+
+    {
+        // Frame 1
+        let frame = &s.frames[1];
+        let valid = &frame.context.valid;
+        assert_eq!(frame.trust, FrameTrust::FramePointer);
+        if let MinidumpContextValidity::Some(ref which) = valid {
+            assert_eq!(which.len(), 4);
+        } else {
+            unreachable!();
+        }
+
+        if let MinidumpRawContext::Arm64(ctx) = &frame.context.raw {
+            assert_eq!(ctx.get_register("pc", valid).unwrap(), return_address1);
+            assert_eq!(ctx.get_register("lr", valid).unwrap(), return_address2);
+            assert_eq!(
+                ctx.get_register("sp", valid).unwrap(),
+                frame1_sp.value().unwrap()
+            );
+            assert_eq!(
+                ctx.get_register("fp", valid).unwrap(),
+                frame1_fp.value().unwrap()
+            );
+        } else {
+            unreachable!();
+        }
+    }
+
+    // Never get to frame 2, alas!
+}
